(
	/*
	NAME: makeTX batcher script
	GOALS: 
	*) collect textures from the user's selected folder and subfolders
	*) Build a batch of the commands
	*) Execute the batch and convert the input textures to .TX
	*/
	
	-- Globals
	global txConverterRoll
	try (destroyDialog txConverterRoll) catch()
	
	-- Locals
	mapFilePaths=#()	--this array will contain the textures
	mapExtensions=#("*.bmp","*.cin","*.dds","*.dpx","*.exr","*.hdr","*.jpg","*.jpeg","*.png","*.ptx","*.rla","*.sgi","*.pic","*.tga","*.tif","*.tiff")	--this array contains the string patterns for the texture extensions. More can be added, or some removed, compatibly with makeTX.exe. Check "https://github.com/OpenImageIO/oiio/blob/release/src/doc/openimageio.pdf" for more info.
	rootFolder=undefined	--the root folder to search textures into. Initialises as "undefined", and cancelling out of the folder picking will set it again to "undefined".
	makeTXPath=undefined	--the makeTX.exe file path. Will be filled later if the file is found.
	optionsSet=#(" --hicomp"," --tile 64 64", " --unpremult")		--array containing options, WITH leading space. f.e. #(" --hicomp" ). Off currently.
	totCounter = 0 		--total file counter
	iniFilePath = getDir #plugcfg + @"\txConverterSettings.ini"
	VRayMajorVersion = undefined

	--Locals for various options
	doUnpremult = false
	doHiComp = false
	
	fn saveIniFile inArray =
	(
		if inArray[1] do
		(
			if makeTXPath != undefined and  makeTXPath != "" do 
			(
				setINISetting iniFilePath "Defaults" "makeTXPath" makeTXPath
			)
		)
		if inArray[2] do
		(
			if rootFolder != undefined and rootFolder != "" do
			(
				setINISetting iniFilePath "Defaults" "scanRoot" rootFolder
			)
		)
	)	
	
	fn getFilesRecursive root pattern =			--function to recursively parse a folder and subfolders to look for specific file types
	(
		root+="/"
		dir_array = #(root)
		join dir_array (GetDirectories (root+"/*") recurse:true)
		my_files = #()
		for f in dir_array do
		  join my_files (getFiles (f + pattern))
		my_files
	)	
	
	fn collectBitmapTextures rootFolder extensions =
	(
		mapFilePaths=#()
		for ex in extensions do
		(
			join mapFilePaths (getFiles (rootFolder+"\\"+ex) recurse:true)
		)
		mapFilePaths
	)	
	
	fn getVRayMajorVersion =
	(
		print "getVRayMajorVersion"
		VRayMajorVersion = execute (vrayVersion())[1][1]
		if VRayMajorVersion != 3 and VRayMajorVersion != 4 and VRayMajorVersion != 5 and VRayMajorVersion != 6 then
			(
				messagebox "Severe Error: cannot determine the currently installed V-Ray Version! Quitting."
				destroyDialog txConverterRoll
			)
		else 
			(
				print ("V-Ray is version " + VRayMajorVersion as string + ".X")
			)
	)

	fn findVRayBinFolder6 =		-- V-Ray 6.x bin folder search
	(
		MaxVer=getMaxReleaseAsString()
		envVar = (systemTools.getEnvVariable ("VRAY_FOR_3DSMAX"+maxVer+"_MAIN"))
			
		theBinFolder = pathConfig.normalizePath envVar + @"\"  
		try
		(
			MaxVer=getMaxReleaseAsString()
			theBinFolder = pathConfig.normalizePath envVar + @"\"  
		)
		catch
		(
			theBinFolder=undefined
		)
-- 		print theBinFolder
		theBinFolder				
	)	

	fn findVRayBinFolder =		-- V-Ray 5.x bin folder search
	(
		MaxVer=getMaxReleaseAsString()
		envVar = (systemTools.getEnvVariable ("VRAY" + (vrayVersion())[1][1] + "_FOR_3DSMAX"+maxVer+"_MAIN"))
		if envVar != undefined then
			(
				theBinFolder = pathConfig.normalizePath envVar + @"\"  
				try
				(
					MaxVer=getMaxReleaseAsString()
					theBinFolder = pathConfig.normalizePath envVar + @"\"  
				)
				catch
				(
					theBinFolder=undefined
				)
-- 				print theBinFolder
				theBinFolder
			)
		else findVRayBinFolder6()
	)	
	
	fn findVRayNextBinFolder =		-- V-Ray Next bin folder search
	(
		Val1=undefined
		maxExtendedVersionNumber="20" + ((maxVersion())[4]-2) as string 
		try
		(
			registry.openKey HKEY_LOCAL_MACHINE "SYSTEM" accessRights:#readOnly key:&key1
			registry.openKey key1 "CurrentControlSet" accessRights:#readOnly key:&key2
			registry.openKey key2 "Control" accessRights:#readOnly key:&key3
			registry.openKey key3 "Session Manager" accessRights:#readOnly key:&key4
			registry.openKey key4 "Environment" accessRights:#readOnly key:&key5
			registry.queryValue key5 ("VRAY4_FOR_3DSMAX" + maxExtendedVersionNumber+"_MAIN") type:#REG_EXPAND_SZ value:&Val1 expand:true 
		)
		catch
		(
			Val1=undefined
		)
		Val1				
	)
	
	fn checkMakeTX5 inPath = -- the function for v5 only checks for the exe, as the libraries are not present
	(
		if matchpattern inpath pattern:"*makeTX.exe" then inpath = getfilenamepath inpath
		if 	inpath != undefined and inpath != "" and
			doesFileExist (inPath+@"\makeTX.exe") 		
		then return true
		else return false
	)		
	
	fn checkMakeTX inPath = -- the function for v4  checks for the exe, as well as the ancillary libraries
	(
		if matchpattern inpath pattern:"*makeTX.exe" then inpath = getfilenamepath inpath
		if 	inpath != undefined and inpath != "" and
			doesFileExist (inPath+@"\makeTX.exe") and
			doesFileExist (inPath+@"\msvcp120.dll") and
			doesFileExist (inPath+@"\msvcr120.dll") and
			doesFileExist (inPath+@"\OpenColorIO.dll") 		
		then return true
		else return false
	)	
	
	fn getMakeTX =	--function to find the path to the makeTX.exe executable file
	(
		print "getmaketx"
		print VRayMajorVersion
		case VRayMajorVersion of
		(
			6: 
			(
				print "V-Ray Version 6.X"
				makeTXPath = findVRayBinFolder6()
				print makeTXPath
				if checkMakeTX5 makeTXPath then
				(
					makeTXPath+=@"makeTX.exe"
					saveINIfile #(true,false)
				)
				else
				(
					VRayMajorVersion = 3 --attempt to manually set path. Do not error out
					getMakeTX()
				)
			)
			5: 
			(
				print "V-Ray Version 5.X"
				makeTXPath = findVRayBinFolder()
				if checkMakeTX5 makeTXPath then
				(
					makeTXPath+=@"makeTX.exe"
					saveINIfile #(true,false)
				)
				else
				(
					VRayMajorVersion = 3 --attempt to manually set path. Do not error out
					getMakeTX()
				)
			)
			4:	--Next: find bin folder and makeTX there.
			(
				print "V-Ray Version 4.X"
				makeTXPath = findVRayBinFolder()
				if checkMakeTX makeTXPath then
				(
					makeTXPath+=@"makeTX.exe"
					saveINIfile #(true,false)
				)
				else
				(
					VRayMajorVersion = 3 --attempt to manually set path. Do not error out
					getMakeTX()
				)
			)
			3:	--3.x, manual Path input.
			(
				print "wtf!"
				fullInPath = (getOpenFileName caption:"Pick the folder where MakeTX.exe and libraries are:" filename:"MakeTX.exe" )
				if fullInPath != undefined then 
				(
					inPath =getfilenamepath fullInPath
					if checkMakeTX inPath then
					(
						makeTXPath = inPath + @"makeTX.exe"
						saveINIfile #(true,false)
					)
					else
					(
						if (queryBox "MakeTX.exe not found or valid. Would you like to try again?\nPressing NO will quit the script") then
						(
							VRayMajorVersion = 3	--redundant, but for the sake of code symmetry.
							getMakeTX()
						)
						else
						(
							VRayMajorVersion = 0	--error!
							getMakeTX()						
						)
					)
				)
				else
				(
					if (queryBox "MakeTX.exe not found or valid. Would you like to try again?\nPressing NO will quit the script") then
					(
						VRayMajorVersion = 3	--redundant, but for the sake of code symmetry.
						getMakeTX()
					)
					else
					(
						VRayMajorVersion = 0	--error!
						getMakeTX()						
					)					
				)
			)
			0: --error.
			(
				messageBox "Couldn't Find MakeTX.exe. Quitting."
				destroyDialog txConverterRoll
			)
			(-1):	--read from INI file.
			(
				makeTXPath
			)
		)
	)
	
	fn setRootFolder =
	(
		rootFolder = getSavePath caption:"Select a root folder to parse for textures" initialDir:(if rootFolder != undefined then rootFolder else #Maps)
		if rootFolder != undefined and rootFolder != "" do
		(
			saveINIfile #(false,true)
		)
	)
	
	fn getINIfile inArray= 		
	(
		if inArray[1] do  --deprecated
		(
			makeTXPathTMP = getINISetting iniFilePath "Defaults" "makeTXPath" 

			if makeTXPathTMP != undefined and  makeTXPathTMP != "" do 
			(
				print makeTXPathTMP
				if checkMakeTX makeTXPathTMP then --files exists, no need for more work.
				(
					print "valid"
					makeTXPath = makeTXPathTMP
					VRayMajorVersion = getVRayMajorVersion()
				)
				else
				(
					print "invalid"
					makeTXPath = undefined 	--in case the file was moved or deleted bewteen sessions.
					VRayMajorVersion = getVRayMajorVersion()
					getMakeTX()
				)
			)
		)
		if inArray[2] do
		(
			if rootFolder != undefined and rootFolder != "" do
			(
				rootFolder = getINISetting iniFilePath "Defaults" "scanRoot" 
			)
		)
	)	
	
	fn mainFunction =
	(
		for f=1 to totCounter do 
		(
			windows.processPostedMessages()
			baseCmd = ("\""+makeTXPath + "\" \"" + mapFilePaths[f]+"\"")
			if doHiComp do baseCmd += optionsSet[1]
			if doUnpremult do baseCmd += optionsSet[3]
			print baseCmd
			percent = ((f as float / totCounter)*100.)
			print ("Converting " + mapFilePaths[f] + "; number "+ f as string + " of " + totCounter as string + "; " + percent as string + "% done.")
			txConverterRoll.pb1.value = percent
			txConverterRoll.lbl2.text = (percent as string + "% done")
			newCol = txConverterRoll.pb1.color
			newCol.h=96.0*percent/100.0
			txConverterRoll.pb1.color =  newCol
			vrayshellLaunch baseCmd waitExec:true
		)
	)

	rollout txConverterRoll ".TX converter v 0.06" width:384 height:400
	(
		GroupBox 'grp1' "Script Config:" pos:[8,8] width:368 height:96
		edittext 'edt1' "" pos:[8,56] width:360 height:16 enabled:false
		label 'lbl3' "MakeTX.exe found in:" pos:[24,32] width:328 height:16
		

		GroupBox 'grp3' "MakeTx Options" pos:[8,112] width:368 height:40
		checkbox 'chk1' "Unpremult" pos:[16,128] width:80 height:16 tooltip:"Unpremultiplies and premultiplies back alpha-containing images. NOT ADAPTIVE."
		checkbox 'chk2' "HiComp" pos:[104,128] width:80 height:16 tooltip:"Tonally compresses input images to expand them after the process. Useful to avoid ringing in MIPs in HDR images."

		GroupBox 'grp2' "Script Operation:" pos:[8,160] width:368 height:232
		button 'btn1' "Pick Root to scan for textures" pos:[16,176] width:352 height:40
		label 'lbl1' "Info" pos:[16,224] width:352 height:48
		button 'btn2' "Go" pos:[16,272] width:352 height:64 enabled:false
		progressBar 'pb1' "ProgressBar" pos:[16,368] width:352 height:16 color:(color 255 0 0)
		label 'lbl2' "" pos:[16,344] width:352 height:16 align:#left
		button 'btn3' "Reset Folders Defaults" pos:[16,80] width:352 height:16 toolTip:"Reset INI file"
		
		on txConverterRoll open do
		(
			if doesFileExist iniFilePath then
			(
				getINIfile #(false,true) --won't load the makeTX path from ini file anymore. only the texture scan path.
				getVRayMajorVersion()
				getMakeTX()
			)
			else
			(
				getVRayMajorVersion()
				getMakeTX()
			)
			if makeTXPath != undefined do edt1.text = makeTXPath
			pb1.color.h=0
		)
		
		on chk1 changed state do
		(
			if state do messageBox "This is not adaptive. Please ensure it's turned on *only* to convert premultiplied images."
			doUnpremult = state
		)
		
		on btn1 pressed do
		(
			setRootFolder()
			if rootFolder!=undefined then
			(
				btn1.text = rootFolder
				lbl1.text="Scanning the root and subfolders for texture files.\nPlease be patient."
				collectBitmapTextures rootFolder mapExtensions
				totCounter = mapFilePaths.count
				if totCounter != 0 then
				(
					lbl1.text = ("Found " + totCounter as string + " texture files.")
					btn2.enabled=true
				)
				else
				(
					lbl1.text = ("Found no texture files. Please choose a different folder.")
					btn2.enabled=false
				)
			)
			else
			(
				btn1.text="Pick Root to scan for textures"
				lbl1.text=""
				btn2.enabled=false
			)
		)
		
		on btn2 pressed do
		(
			mainFunction()
			lbl1.text="Conversion attempt completed.\nThe MaxScript Listener will have some more information.\nPlease verify the results!"
			lbl2.text=""
			btn2.enabled=false
		)
		
		on btn3 pressed do
		(
			try (deletefile iniFilePath) catch()
			destroyDialog txConverterRoll
			createdialog txConverterRoll
		)
		
		on chk2 changed state do
		(
			doHiComp = state
		)
	)
	
	createdialog txConverterRoll
	
)
